//
//  GameView.m
//  T4Two
//

#import "GameView.h"

#import <GraphicsServices/GraphicsServices.h>
#import <UIKit/UIView-Geometry.h>
#import <UIKit/UIView-Gestures.h>
#import <UIKit/UIView-Animation.h>

#ifdef ASPEN

#ifndef UIKIT_UIFont_UIColor_H
#define UIKIT_UIFont_UIColor_H
typedef float CGFloat;
#import <UIKit/UIFont.h>
#import <UIKit/UIColor.h>
#endif

#ifndef UIKIT_UIEvent_UITouch_H
#define UIKIT_UIEvent_UITouch_H
typedef unsigned int NSUInteger;
#import <UIKit/UIEvent.h>
#import <UIKit/UITouch.h>
#endif

extern UIApplication *thisApp;

#endif

#ifndef ASPEN
extern CGPoint GSEventGetLocationInWindow(struct __GSEvent*);
extern CGPoint GSEventGetInnerMostPathPosition(struct __GSEvent*);
extern CGPoint GSEventGetOuterMostPathPosition(struct __GSEvent*);
#endif

#define	kMaxScore			10
#define	kDefaultInterval	0.02f
#define	kServeInterval		0.5f

@implementation GameView

//-----------------------------------------------------------------------

// ビューの初期化
-(id)initWithFrame:(CGRect)rect
{
	LogMethod();	
	int	i;
	CGRect	imageRect;
	
	// ビューの初期化
	self = [super initWithFrame:rect];
	paused = NO;
	debuginfo = YES;
	sound = NO;
	// スコア・バーを作成	// autoreleaseしない
	barImage[0] = [UIImage imageNamed:@"bar-0.png"];
	barImage[1] = [UIImage imageNamed:@"bar-1.png"];
	barImage[2] = [UIImage imageNamed:@"bar-2.png"];
	imageRect.size = [barImage[0] size];	// 20 x 48
	for(i=0; i<10; i++)
	{
		imageRect.origin.x = 0;
		imageRect.origin.y = i * imageRect.size.height;
		barView[0][i] = [[[UIImageView alloc] initWithFrame:imageRect] autorelease];
		[barView[0][i] setImage:barImage[0]];
		[barView[0][i] setEnabled:NO];
		[self addSubview:barView[0][i]];		
	}
	for(i=0; i<10; i++)
	{
		imageRect.origin.x = 320 - imageRect.size.width;
		imageRect.origin.y = 480 - ((i+1) * imageRect.size.height);
		barView[1][i] = [[[UIImageView alloc] initWithFrame:imageRect] autorelease];
		[barView[1][i] setImage:barImage[1]];
		[barView[1][i] setEnabled:NO];
		[self addSubview:barView[1][i]];		
	}
	

	for(i=0; i<=3; i++)
	{
		label[i] = [[UITextLabel alloc] initWithFrame: CGRectMake(0, 120+i*20, 320, 20)];
		[label[i] setFont:[UIFont fontWithName:@"Marker Felt" size:14]];
		[label[i] setCentersHorizontally: YES];
		[label[i] setText:@""];
		[label[i] setBackgroundColor:[UIColor clearColor]];
		[label[i] setColor:[UIColor whiteColor]];
		[self addSubview:label[i]];
		label[i].hidden = !debuginfo;
	}
	[label[3] setColor:[UIColor yellowColor]];

#ifdef ENABLE_ACCELEROMETER
	xarrow = [[UIImageView alloc] initWithFrame: CGRectMake(135.0, 280.0, 50.0, 46.0)];
	UIImage *img = [[UIImage imageAtPath:@"/Applications/MobileNotes.app/arrow right.png"] retain];
    [xarrow setImage:img];
//	[xarrow setTransform:CGAffineTransformInvert(CGAffineTransformIdentity)];
	[self addSubview:xarrow];
	xarrow.hidden = !debuginfo;
#endif

	
	// ボールの位置を設定
	ballPosition = CGPointMake(160.0f, 240.0f);
	// ボール・ビューを作成
	ballView = [[[UIImageView alloc] initWithImage:[UIImage imageNamed:@"ball.png"]] autorelease];
	[ballView setPosition:ballPosition];
	[ballView setEnabled:NO];
	[self addSubview:ballView];
	ballRadius = [ballView frame].size.width * 0.5;	// 10
	
	// 指のエリアと位置を設定
	fingerArea[0] = CGRectMake(0, 0, 320, 100);
	fingerArea[1] = CGRectMake(0, 380, 320, 100);
	fingerPosition[0] = CGPointMake(160, 10);	// y値は使用しない
	fingerPosition[1] = CGPointMake(160, 470);	// y値は使用しない

	// ラケットの位置を設定
	racketPosition[0] = CGPointMake(160, 65);
	racketPosition[1] = CGPointMake(160, 415);
	// ラケット・ビューを作成　ラケット0は赤色、上部
	racketView[0] = [[[UIImageView alloc] initWithImage:[UIImage imageNamed:@"racket-0.png"]] autorelease];
	[racketView[0] setPosition:racketPosition[0]];
	[racketView[0] setEnabled:NO];
	[self addSubview:racketView[0]];
	// ラケット・ビューを作成　ラケット1は青色、下部
	racketView[1] = [[[UIImageView alloc] initWithImage:[UIImage imageNamed:@"racket-1.png"]] autorelease];
	[racketView[1] setPosition:racketPosition[1]];
	[racketView[1] setEnabled:NO];

	[self addSubview:racketView[1]];
	
	// ジェスチャー処理を有効化
	// メイン・ビュー以外はsenEnabled:NOにしている→メイン・ビューがイベント処理をする
//        [ self setTapDelegate: self ];
#ifdef ASPEN
//        [ self setEnabled: YES ];
//        [ self setEnabledGestures: 1 ];
//        [ self setGestureDelegate: self ];
		[ self becomeFirstResponder];
#endif

	
	// サウンド再生を準備
	av = [[AVController alloc] init];
	avQueue = [[AVQueue alloc] init];
	[self loadSound:@"game_serve"];
	[self loadSound:@"game_over"];
	[self loadSound:@"hit_on_wall"];
	[self loadSound:@"hit_on_racket-0"];
	[self loadSound:@"hit_on_racket-1"];
	
	return	self;
}

- (BOOL)ignoresMouseEvents {

    return NO;
}

- (BOOL)canHandleGestures {

   return YES;
}


// ビューの消滅時の処理
- (void)dealloc
{
	LogMethod();
	int	i;
	
	[ballView release];

	for(i=0; i<2; i++)
		[barImage[i] release];
		
	for(i=0; i<1; i++)
		[racketView[i] release];

	for(i=0; i<10; i++)
	{
		[barView[0][i] release];
		[barView[1][i] release];
	}


#ifdef ENABLE_ACCELEROMETER	
	[xarrow release];
#endif
	for(i=0; i<=3; i++)
		[label[i] release];

	[av release];
	[avQueue release];

	[super dealloc];
}

//-----------------------------------------------------------------------

// ゲームの開始
- (void)gameStart
{
	LogMethod();
	int	i;
	NSLog(@"gameStart");
	
	srand(time(NULL));
	score[0] = score[1] = 0;
	for(i=0; i<10; i++)
	{
		[barView[0][i] setImage:barImage[2]];
		[barView[1][i] setImage:barImage[2]];
	}
	[self ballReset];
	[self gameLoop:nil];
}

-(void)gameSuspend
{
	paused = YES;
}

-(void)gameResume
{
	if (paused)
	{
		paused = NO;
		[self gameLoop:nil];
	}
}

//-----------------------------------------------------------------------

// ゲーム中の処理
- (void)gameLoop:(id)argument
{
	//LogMethod();	
	int	i;
	CGPoint	prevPosition;

	// 繰り返し時間間隔をデフォルト値に設定
	interval = kDefaultInterval;

	// ラケット（ビュー）の位置を更新
	for(i=0; i<2; i++)
	{
		if (fingerPosition[i].x != racketPosition[i].x)
		{
			racketPosition[i].x = racketPosition[i].x * 0.5 + fingerPosition[i].x * 0.5;
			[racketView[i] setPosition:racketPosition[i]];
			//[racket setNeedsDisplay];	// これは必要ないみたい
		}
	}

	// ボールの直前の位置を記憶
	prevPosition = ballPosition;
	// ボールの新しい位置を設定
	ballPosition.x += ballVector.x;
	ballPosition.y += ballVector.y;

	// 左の壁との衝突判定＆処理
	if( ballPosition.x > 320 - 20 - ballRadius )
	{
		[self playSound:SoundHitWall];
		ballPosition.x = 320 - 20 - ballRadius;
		ballVector.x *= -ballAccel;
		ballVector.y *= ballAccel;
	}
	
	// 右の壁との衝突判定＆処理
	if( ballPosition.x < ballRadius + 20 )
	{
		[self playSound:SoundHitWall];
		ballPosition.x = ballRadius + 20;
		ballVector.x *= -ballAccel;
		ballVector.y *= ballAccel;
	}
	
	// TODO: ラケットの動かし方や打点位置に応じた跳ね返り方の工夫が必要？
	// ラケット0との衝突判定＆処理
	if (prevPosition.y - ballRadius >= racketPosition[0].y + 5
	&&  ballPosition.y - ballRadius <= racketPosition[0].y + 5)
	{
		if (ballPosition.x + ballRadius >= racketPosition[0].x - 40
		&&  ballPosition.x - ballRadius <= racketPosition[0].x + 40)
		{
			[self playSound:SoundHitRacket0];
			ballPosition.y = racketPosition[0].y + 5 + ballRadius;
			ballVector.x *= ballAccel;
			ballVector.y *= -ballAccel;

			float diff = abs(ballPosition.x - racketPosition[0].x) / 40.0f;
			ballVector.x *= 1.0f + ((rand() % 200 - 100) * 0.01f * diff);
		}
	}
	
	// ラケット1との衝突判定＆処理
	if (prevPosition.y + ballRadius <= racketPosition[1].y - 5
	&&  ballPosition.y + ballRadius >= racketPosition[1].y - 5)
	{
		if (ballPosition.x + ballRadius >= racketPosition[1].x - 40
		&&  ballPosition.x - ballRadius <= racketPosition[1].x + 40)
		{
			[self playSound:SoundHitRacket1];
			ballPosition.y = racketPosition[1].y - 5 - ballRadius;
			ballVector.x *= ballAccel;
			ballVector.y *= -ballAccel;
			
			float diff = abs(ballPosition.x - racketPosition[1].x) / 40.0f;
			ballVector.x *= 1.0f + ((rand() % 200 - 100) * 0.01f * diff);
		}
	}
	
	// ボールの上端への通過判定＆処理
	if( ballPosition.y < -ballRadius )
		[self gameScore:1];	
	// ボールの下端への通過判定＆処理
	if( ballPosition.y > 480 + ballRadius)
		[self gameScore:0];
	
	// ボール（ビュー）の位置を更新
	[ballView setPosition:ballPosition];
	//[racket setNeedsDisplay];	// これは必要ないみたい
		
	// ゲーム・オーバーを判定
	if (score[1] >= kMaxScore || score[0] >= kMaxScore)
	{
		[self playSound:SoundGameOver];
		[self gameRetry];
	}
	else
	{
		if (!paused)
			[self performSelector: @selector(gameLoop:) withObject: nil afterDelay:interval];
	}
}


//-----------------------------------------------------------------------

// ボールの情報を初期化
- (void)ballReset
{
	//LogMethod();

	ballAccel = 1.05;
	ballVector.x = (rand() % 200 - 100) * 0.01;
	ballVector.y = rand() % 2 ? 3.0f : -3.0f;
	ballPosition = CGPointMake(160.0f, 240.0f);

	[self playSound:SoundGameServe];
	interval = kServeInterval;
}

// 得点を得た処理
- (void)gameScore:(int)player
{
	//LogMethod();
	
	[barView[player][score[player]] setImage:barImage[player]];
	score[player] += 1;
	
	[self ballReset];		
}

// 勝敗を表示し、ゲームのリトライを尋ねる
- (void)gameRetry
{
	LogMethod();
	
	NSArray *buttons = [NSArray arrayWithObjects:@"Yes", @"No", nil];
	UIAlertSheet *alertSheet = [[UIAlertSheet alloc] initWithTitle:@"Tennis for Two" buttons:buttons defaultButtonIndex:1 delegate:self context:self];
	if (score[0] == 0 && score[1] == 0)
	{
		[alertSheet setBodyText:@"Ready?"];
	}
	else
	{
		if (score[0] > score[1])
			[alertSheet setBodyText:@"Red Wins.\nPlay again?"];
		else
			[alertSheet setBodyText:@"Blue Wins.\nPlay again?"];

	}
	//[alertSheet setRunsModal:YES];
	[alertSheet popupAlertAnimated:YES];
	[alertSheet becomeFirstResponder];
}

// OKボタンが押されると、ゲームを開始する
- (void)alertSheet:(UIAlertSheet *)sheet buttonClicked:(int)button
{
	LogMethod();
	
	[sheet dismissAnimated:YES];
	[sheet release];
	
	if (button == 1)
		[self gameStart];
	else
#ifdef ASPEN
		[self terminate];
#else
		[self terminateWithSuccess];
#endif
}

#ifdef ASPEN

-(void) terminate
{
	[thisApp terminate];
}

#endif

//-----------------------------------------------------------------------
#ifdef ASPEN



- (void)touchesChangedWithEvent:(UIEvent *)event
{
	NSSet *touches = [event allTouches];
    NSUInteger numTaps = [[touches anyObject] tapCount];
	int touchIndex = 0;
	if (debuginfo) {
		int touchCount = [touches count];
		[label[0] setText:[NSString stringWithFormat:@"Touches = %d",touchCount]];
		[label[0] setColor:[UIColor whiteColor]];
		[label[1] setText:@""];
		[label[2] setText:@""];
	}
	else {
		label[0].hidden = label[1].hidden = label[2].hidden = label[3].hidden = YES;
	}
	for (UITouch *touch in touches) {
		if (debuginfo) {
			touchIndex++;
		}
		if (touch.phase == UITouchPhaseMoved) {
			if (debuginfo) {
				CGPoint p = [touch locationInView:self];
				CGPoint lp = [touch previousLocationInView:self];
				if (touchIndex < 3)
					[label[touchIndex] setText:[NSString stringWithFormat:@"TouchMoved %d = %3.0f, %3.0f ; last = %3.0f, %3.0f", touchIndex, p.x, p.y, lp.x, lp.y ]];
			}
			[self dispatchTouchEvent:[touch view] toPosition:[touch locationInView:self]];
		}
		else if (touch.phase == UITouchPhaseBegan) {
			if (debuginfo) {
				CGPoint p = [touch locationInView:self];
				CGPoint lp = [touch previousLocationInView:self];
				if (touchIndex < 3)
					[label[touchIndex] setText:[NSString stringWithFormat:@"TouchBegan %d = %3.0f, %3.0f ; last = %3.0f, %3.0f", touchIndex, p.x, p.y, lp.x, lp.y ]];
			}
			[self dispatchTouchEvent:[touch view] toPosition:[touch locationInView:self]];
			if(numTaps >= 2) {
				if (numTaps == 3) {  //triple tap
					sound = !sound;
					[label[0] setColor:[UIColor magentaColor]];
					[label[0] setText:[NSString stringWithFormat:@"Sound = %@",sound?@"ON":@"OFF"]];

				}
				else if (numTaps == 2) {  //double tap
					debuginfo = !debuginfo;
					[label[0] setColor:[UIColor magentaColor]];
					[label[0] setText:[NSString stringWithFormat:@"Debug = %@",debuginfo?@"ON":@"OFF"]];
					label[0].hidden = label[1].hidden = label[2].hidden = label[3].hidden = !debuginfo;
#ifdef ENABLE_ACCELEROMETER	
					xarrow.hidden = !debuginfo;
#endif
				}
			} 	
		}
		else if (touch.phase == UITouchPhaseEnded) {
			if (debuginfo) {
				CGPoint p = [touch locationInView:self];
				CGPoint lp = [touch previousLocationInView:self];
				if (touchIndex < 3)
					[label[touchIndex] setText:[NSString stringWithFormat:@"TouchEnded %d = %3.0f, %3.0f ; last = %3.0f, %3.0f", touchIndex, p.x, p.y, lp.x, lp.y ]];
			}
		}
		else if (touch.phase == UITouchPhaseStationary) {
			if (debuginfo) {
				CGPoint p = [touch locationInView:self];
				CGPoint lp = [touch previousLocationInView:self];
				if (touchIndex < 3)
					[label[touchIndex] setText:[NSString stringWithFormat:@"TouchStationary %d = %3.0f, %3.0f ; last = %3.0f, %3.0f", touchIndex, p.x, p.y, lp.x, lp.y ]];
			}
			[self dispatchTouchEvent:[touch view] toPosition:[touch locationInView:self]];
		}
		else if (touch.phase == UITouchPhaseCanceled) {
			if (debuginfo) {
				CGPoint p = [touch locationInView:self];
				CGPoint lp = [touch previousLocationInView:self];
				if (touchIndex < 3)
					[label[touchIndex] setText:[NSString stringWithFormat:@"TouchCanceled %d = %3.0f, %3.0f ; last = %3.0f, %3.0f", touchIndex, p.x, p.y, lp.x, lp.y ]];
			}
		}
	}
			
}

// タップされた位置によってラケットを移動する
-(void) dispatchTouchEvent:(UIView *)theView toPosition:(CGPoint)p
{
//	NSLog(@"dispatchTouchEvent %f",p.y);

	// Check to see which view, or views,  the point is in and then move to that position.
	if (CGRectContainsPoint(fingerArea[0], p))
			fingerPosition[0].x = p.x;
	else
	if (CGRectContainsPoint(fingerArea[1], p))
		fingerPosition[1].x = p.x;
}


#else
- (void)mouseUp:(GSEvent *)e
{
	//LogMethod();
}

- (void)mouseDown:(GSEvent *)e
{
	NSLog(@"mouseDown");
	//LogMethod();
	CGPoint	p = GSEventGetLocationInWindow(e);
	//NSLog(@"p.x=$f p.y=%f",p.x, p.y);
	[self moveRacket:p];
}

- (void)mouseDragged:(GSEvent *)e
{
	NSLog(@"mouseDragged");
	//LogMethod();
	CGPoint	p = GSEventGetLocationInWindow(e);
	//NSLog(@"p.x=$f p.y=%f",p.x, p.y);
	[self moveRacket:p];
}

- (void)gestureStarted:(struct __GSEvent *)e
{	
	//LogMethod();
}

- (void)gestureEnded:(struct __GSEvent *)e
{
	//LogMethod();
}

- (void)gestureChanged:(struct __GSEvent *)e
{
	//LogMethod();
	//
	CGPoint p0 = GSEventGetInnerMostPathPosition(e);
	CGPoint p1 = GSEventGetOuterMostPathPosition(e);
	//NSLog(@"p0.x=$f p0.y=%f p1.x=%f p1.y=%f",p0.x, p0.y, p1.x, p1.y);
	[self moveRacket:p0];
	[self moveRacket:p1];
}

// タップされた位置によってラケットを移動する
- (void)moveRacket:(CGPoint)p
{
	if (CGRectContainsPoint(fingerArea[0], p))
		fingerPosition[0].x = p.x;
	else
	if (CGRectContainsPoint(fingerArea[1], p))
		fingerPosition[1].x = p.x;
}

#endif
//-----------------------------------------------------------------------


//-----------------------------------------------------------------------
#pragma mark === Responding to accelerations ===

// 加速度センサーによってボールの動きを変える
- (void)acceleratedInX:(float)x Y:(float)y Z:(float)z
{
	ballVector.x += x * -3.0f;
	ballVector.y += y * -3.0f;

	if (debuginfo) {
		[label[3] setText:[NSString stringWithFormat:@"Ball speed = %3.0f, %3.0f", ballVector.x, ballVector.y ]];
	}
#if defined ASPEN && defined ENABLE_ACCELEROMETER
   // Revert Arrow and then rotate to new coords
   [xarrow setTransform:CGAffineTransformIdentity];
   float angle = atan2(y, x);
   angle *= 180.0/3.14159;
   [xarrow setRotationBy:-angle];
#endif
}
//-----------------------------------------------------------------------

// サウンドを読み込む
- (void)loadSound:(NSString *)soundName
{
	//LogMethod();
	AVItem	*avItem;
	
	NSString *soundPath = [[NSBundle mainBundle] pathForResource:soundName ofType:@"wav" inDirectory:@"/"];
	avItem = [[AVItem alloc] initWithPath:soundPath error:nil];
	[avQueue appendItem:avItem error:nil];
	[avItem release];
}

// サウンドを鳴らす
- (void)playSound:(SoundIndex)index
{
	//LogMethod();
	if (sound) {
	//[av setCurrentItem:avItem[index]];
		[av setCurrentItem:[avQueue itemAtIndex:index]];
		[av play:nil];
	}
}

//-----------------------------------------------------------------------

@end
