//
//  common.h
//  workbench
//

#define ENABLE_ACCELEROMETER
#undef ENABLE_ACCELEROMETER

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

#ifdef ASPEN

#ifndef UIKIT_UIView_property_H
#define UIKIT_UIView_property_H

#ifndef UIKIT_UIFont_UIColor_H
#define UIKIT_UIFont_UIColor_H
typedef float CGFloat;
#import <UIKit/UIFont.h>
#import <UIKit/UIColor.h>
#endif

#import<UIKit/UIView.h>

typedef int NSInteger;

typedef enum {
    UIViewAnimationCurveEaseInOut,         // slow at beginning and end
    UIViewAnimationCurveEaseIn,            // slow at beginning
    UIViewAnimationCurveEaseOut,           // slow at end
    UIViewAnimationCurveLinear
} UIViewAnimationCurve;

typedef enum {
    UIViewContentModeScaleToFill,
    UIViewContentModeScaleAspectFit,      // contents scaled to fit with fixed aspect. remainder is transparent
    UIViewContentModeScaleAspectFill,     // contents scaled to fill with fixed aspect. some portion of content may be clipped.
    UIViewContentModeRedraw,              // redraw on bounds change (calls -setNeedsDisplay)
    UIViewContentModeCenter,              // contents remain same size. positioned adjusted.
    UIViewContentModeTop,
    UIViewContentModeBottom,
    UIViewContentModeLeft,
    UIViewContentModeRight,
    UIViewContentModeTopLeft,
    UIViewContentModeTopRight,
    UIViewContentModeBottomLeft,
    UIViewContentModeBottomRight,
} UIViewContentMode;

typedef enum {
    UIViewAnimationTransitionNone,
    UIViewAnimationTransitionFlipFromLeft,
    UIViewAnimationTransitionFlipFromRight
} UIViewAnimationTransition;

enum {
    UIViewAutoresizingNone                 = 0,
    UIViewAutoresizingFlexibleLeftMargin   = 1 << 0,
    UIViewAutoresizingFlexibleWidth        = 1 << 1,
    UIViewAutoresizingFlexibleRightMargin  = 1 << 2,
    UIViewAutoresizingFlexibleTopMargin    = 1 << 3,
    UIViewAutoresizingFlexibleHeight       = 1 << 4,
    UIViewAutoresizingFlexibleBottomMargin = 1 << 5
};

@interface UIView(UIResponder)

@property(getter=isUserInteractionEnabled)  BOOL      userInteractionEnabled;  // default is YES. if set to NO, user events (touch, keys) are ignored and removed from the event queue.
@property                                   NSInteger tag;                     // default is 0
//@property(readonly, retain)                 CALayer  *layer;                   // returns view's layer. Will always return a non-nil value. view is layer's delegate

@end


@interface UIView(UIViewGeometry)
// animatable. do not use frame if view is transformed since it will not correctly reflect the actual location of the view. use bounds + center instead.
@property CGRect            frame;

// use bounds/center and not frame if non-identity transform. if bounds dimension is odd, center may be have fractional part
@property CGRect            bounds;      // default bounds is zero origin, frame size. animatable
@property CGPoint           center;      // center is center of frame. animatable
@property CGAffineTransform transform;   // default is CGAffineTransformIdentity. animatable

@property(getter=isMultipleTouchEnabled) BOOL   multipleTouchEnabled;   // default is NO
@property(getter=isExclusiveTouch) BOOL         exclusiveTouch;         // default is NO
@property BOOL               autoresizesSubviews; // default NO. if set, subviews are adjust if self bounds change
//@property UIViewAutoresizing autoresizingMask;    // simple resize. default is UIViewAutoresizingNone
@end

@interface UIView(UIViewHierarchy)
@property(readonly) UIView              *superview;
@property(readonly, copy) NSArray       *subviews;
@property(readonly) UIWindow            *window;
@end

@interface UIView(UIViewRendering)
@property                  BOOL              clipsToBounds;              // When YES, content and subviews are clipped to the bounds of the view. Default is NO.
@property(retain)          UIColor          *backgroundColor;            // animatable. default is nil
@property                  CGFloat           alpha;                      // animatable. default is 1.0
@property(getter=isOpaque) BOOL              opaque;                     // if opaque, assumes that draw will fill (implies clearsContextBeforeDrawing). default is YES
@property                  BOOL              clearsContextBeforeDrawing; // says we draw full contents even if not opaque. otherwise context is cleared causing extra work. default is YES
@property(getter=isHidden) BOOL              hidden;                     // default is NO. doesn't check superviews
@property                  UIViewContentMode contentMode;                // default is UIViewContentModeScaleToFill
@end
#endif

#include <math.h>

#endif

//From Aaron hillegass
#define LogMethod() NSLog(@"-[%@ %s]", self, _cmd)

typedef enum {
	kUIControlEventMouseDown = 1 << 0,
	kUIControlEventMouseMovedInside = 1 << 2,
	kUIControlEventMouseMovedOutside = 1 << 3,
	kUIControlEventMouseUpInside = 1 << 6,
	kUIControlEventMouseUpOutside = 1 << 7,
	kUIControlEventMouseMoved = (kUIControlEventMouseMovedInside | kUIControlEventMouseMovedOutside),
	kUIControlEventMouseU = (kUIControlEventMouseUpInside | kUIControlEventMouseUpOutside),
	kUIControlAllEvents = (kUIControlEventMouseDown | kUIControlEventMouseMovedInside | kUIControlEventMouseMovedOutside | kUIControlEventMouseUpInside | kUIControlEventMouseUpOutside)
} UIControlEventMasks;

typedef enum
{
	UITransitionImmediate = 0,
	UITransitionShiftLeft = 1,
	UITransitionShiftRight = 2,
	UITransitionShiftUp = 3,
	UITransitionFade = 6,
	UITransitionShiftDown = 7
} UITransitionStyle;
